@@ -6,19 +6,19 @@
 //! ```no_run
 //! #[tokio::main]
 //! async fn main() -> Result<(), avassa_client::Error> {
-//!     use avassa_client::ClientBuilder;
+//!     use avassa_client::Client;
 //!
 //!     // API CA certificate loaded
 //!     let ca_cert = Vec::new();
 //!
 //!     // Use login using platform provided application token
 //!     let approle_id = "secret approle id";
-//!     let client = ClientBuilder::new()
+//!     let client = Client::builder()
 //!         .add_root_certificate(&ca_cert)?
 //!         .application_login("https://api.customer.net", Some(approle_id)).await?;
 //!
 //!     // Username and password authentication, good during the development phase
-//!     let client = ClientBuilder::new()
+//!     let client = Client::builder()
 //!         .add_root_certificate(&ca_cert)?
 //!         .login("https://1.2.3.4", "joe", "secret").await?;
 //!
@@ -31,14 +31,14 @@
 //! ```no_run
 //! #[tokio::main]
 //! async fn main() -> Result<(), avassa_client::Error> {
-//!     use avassa_client::ClientBuilder;
+//!     use avassa_client::Client;
 //!
 //!     // API CA certificate loaded
 //!     let ca_cert = Vec::new();
 //!
 //!     // Use login using platform provided application token
 //!     let approle_id = "secret approle id";
-//!     let client = ClientBuilder::new()
+//!     let client = Client::builder()
 //!         .add_root_certificate(&ca_cert)?
 //!         .application_login("https://api.customer.net", Some(approle_id)).await?;
 //!
@@ -49,10 +49,13 @@
 //!         let mut producer = producer_client.volga_open_producer(
 //!             "test-producer",
 //!             "my-topic",
-//!             Default::default())
+//!             avassa_client::volga::OnNoExists::Create(Default::default())
+//!             )
 //!             .await?;
 //!
-//!         producer.produce("test message").await?;
+//!         producer.produce(&serde_json::json!({
+//!            "msg": "The Message",
+//!         })).await?;
 //!         Ok::<_, avassa_client::Error>(())
 //!     });
 //!
@@ -69,8 +72,10 @@
 //! }
 //! ```
 
-#![deny(missing_docs)]
-
+#![deny(clippy::all)]
+#![warn(clippy::pedantic)]
+#![warn(clippy::cargo)]
+#![allow(clippy::missing_errors_doc)]
 use serde::Deserialize;
 use serde_json::json;
 
@@ -160,6 +165,7 @@ pub enum Error {
 
 impl Error {
     /// Create a general error
+    #[must_use]
     pub fn general(err: &str) -> Self {
         Self::General(err.to_string())
     }
@@ -200,7 +206,8 @@ pub struct ClientBuilder {
 
 impl ClientBuilder {
     /// Create a new builder instance
-    pub fn new() -> Self {
+    #[must_use]
+    const fn new() -> Self {
         Self {
             reqwest_ca: Vec::new(),
             tls_ca: Vec::new(),
@@ -219,6 +226,7 @@ impl ClientBuilder {
     }
 
     /// Disable certificate verification
+    #[must_use]
     pub fn danger_accept_invalid_certs(self) -> Self {
         Self {
             disable_cert_verification: true,
@@ -227,6 +235,7 @@ impl ClientBuilder {
     }
 
     /// Disable hostname verification
+    #[must_use]
     pub fn danger_accept_invalid_hostnames(self) -> Self {
         Self {
             disable_hostname_check: true,
@@ -254,7 +263,7 @@ impl ClientBuilder {
     }
 
     /// Login to an avassa Control Tower or Edge Enforcer instance. If possible,
-    /// please use the application_login as no credentials needs to be distributed.
+    /// please use the `application_login` as no credentials needs to be distributed.
     pub async fn login(&self, host: &str, username: &str, password: &str) -> Result<Client> {
         let base_url = url::Url::parse(host)?;
         let url = base_url.join("v1/login")?;
@@ -266,6 +275,12 @@ impl ClientBuilder {
         });
         Client::do_login(self, base_url, url, data).await
     }
+
+    /// Login using an existing bearer token
+    pub fn token_login(&self, host: &str, token: &str) -> Result<Client> {
+        let base_url = url::Url::parse(host)?;
+        Client::new_from_token(self, base_url, token)
+    }
 }
 
 impl Default for ClientBuilder {
@@ -302,7 +317,8 @@ impl std::fmt::Debug for Client {
 
 impl Client {
     /// Create a Client builder
-    pub fn builder() -> ClientBuilder {
+    #[must_use]
+    pub const fn builder() -> ClientBuilder {
         ClientBuilder::new()
     }
 
@@ -313,17 +329,7 @@ impl Client {
         payload: serde_json::Value,
     ) -> Result<Self> {
         let json = serde_json::to_string(&payload)?;
-        let client = reqwest::Client::builder();
-
-        // Add CA certificates
-        let client = builder
-            .reqwest_ca
-            .iter()
-            .fold(client, |client, ca| client.add_root_certificate(ca.clone()));
-
-        let client = client.danger_accept_invalid_certs(builder.disable_cert_verification);
-
-        let client = client.build()?;
+        let client = Self::reqwest_client(builder)?;
         let result = client
             .post(url)
             .header("content-type", "application/json")
@@ -342,6 +348,35 @@ impl Client {
         }
     }
 
+    fn reqwest_client(builder: &ClientBuilder) -> Result<reqwest::Client> {
+        let reqwest_client_builder = reqwest::Client::builder();
+
+        // Add CA certificates
+        let reqwest_client_builder = builder
+            .reqwest_ca
+            .iter()
+            .fold(reqwest_client_builder, |reqwest_client_builder, ca| {
+                reqwest_client_builder.add_root_certificate(ca.clone())
+            });
+
+        let reqwest_client_builder =
+            reqwest_client_builder.danger_accept_invalid_certs(builder.disable_cert_verification);
+
+        let client = reqwest_client_builder.build()?;
+        Ok(client)
+    }
+
+    fn new_from_token(builder: &ClientBuilder, base_url: url::Url, token: &str) -> Result<Self> {
+        let client = Self::reqwest_client(builder)?;
+        let login_token = LoginToken {
+            token: token.to_string(),
+            expires_in: None,
+            creation_time: None,
+        };
+
+        Self::new(builder, client, base_url, login_token)
+    }
+
     fn new(
         builder: &ClientBuilder,
         client: reqwest::Client,
@@ -402,6 +437,8 @@ impl Client {
     }
 
     /// POST arbitrary JSON to a path
+    /// # Panics
+    /// never
     pub async fn post_json(
         &self,
         path: &str,
@@ -431,7 +468,7 @@ impl Client {
             }
 
             match responses.len() {
-                0 => Ok(serde_json::Value::Object(Default::default())),
+                0 => Ok(serde_json::Value::Object(serde_json::Map::default())),
                 1 => Ok(responses.into_iter().next().unwrap()),
                 _ => {
                     // Convert to a JSON array
@@ -468,14 +505,14 @@ impl Client {
             .send()
             .await?;
 
+        #[allow(clippy::redundant_closure_for_method_calls)]
         if result.status().is_success() {
             use std::error::Error;
             let resp = result.json().await.or_else(|e| match e {
                 e if e.is_decode() => {
                     match e
                         .source()
-                        .map(|e| e.downcast_ref::<serde_json::Error>())
-                        .flatten()
+                        .and_then(|e| e.downcast_ref::<serde_json::Error>())
                     {
                         Some(e) if e.is_eof() => {
                             Ok(serde_json::Value::Object(serde_json::Map::new()))
@@ -497,34 +534,27 @@ impl Client {
         }
     }
 
-    /// Remove me
-    pub fn get_client(&self) -> reqwest::Client {
-        self.client.clone()
-    }
-
     /// Open a volga producer on a topic
     pub async fn volga_open_producer(
         &self,
         producer_name: &str,
         topic: &str,
-        options: volga::Options,
-    ) -> Result<volga::Producer> {
-        crate::volga::ProducerBuilder::new(self, producer_name, topic, options)?
-            .set_options(options)
+        on_no_exists: volga::OnNoExists,
+    ) -> Result<volga::producer::Producer> {
+        crate::volga::producer::Builder::new(self, producer_name, topic, on_no_exists)?
             .connect()
             .await
     }
 
     /// Open a volga NAT producer on a topic in a site
-    pub async fn volga_open_nat_producer(
+    pub async fn volga_open_child_site_producer(
         &self,
         producer_name: &str,
         topic: &str,
         site: &str,
-        options: volga::Options,
-    ) -> Result<volga::Producer> {
-        crate::volga::ProducerBuilder::new_nat(self, producer_name, topic, site, options)?
-            .set_options(options)
+        on_no_exists: volga::OnNoExists,
+    ) -> Result<volga::producer::Producer> {
+        crate::volga::producer::Builder::new_child(self, producer_name, topic, site, on_no_exists)?
             .connect()
             .await
     }
@@ -534,23 +564,23 @@ impl Client {
         &self,
         consumer_name: &str,
         topic: &str,
-        options: crate::volga::ConsumerOptions,
-    ) -> Result<volga::Consumer> {
-        crate::volga::ConsumerBuilder::new(self, consumer_name, topic)?
+        options: crate::volga::consumer::Options,
+    ) -> Result<volga::consumer::Consumer> {
+        crate::volga::consumer::Builder::new(self, consumer_name, topic)?
             .set_options(options)
             .connect()
             .await
     }
 
-    /// Creates and opens a Volga NAT consumer
-    pub async fn volga_open_nat_consumer(
+    /// Creates and opens a Volga consumer on a child site
+    pub async fn volga_open_child_site_consumer(
         &self,
         consumer_name: &str,
         topic: &str,
         site: &str,
-        options: crate::volga::ConsumerOptions,
-    ) -> Result<volga::Consumer> {
-        crate::volga::ConsumerBuilder::new_nat(self, consumer_name, topic, site)?
+        options: crate::volga::consumer::Options,
+    ) -> Result<volga::consumer::Consumer> {
+        crate::volga::consumer::Builder::new_child(self, consumer_name, topic, site)?
             .set_options(options)
             .connect()
             .await
@@ -571,13 +601,18 @@ impl Client {
         let connector: tokio_native_tls::TlsConnector = connector.into();
         let addrs = self.websocket_url.socket_addrs(|| None)?;
         let stream = tokio::net::TcpStream::connect(&*addrs).await?;
-        let stream = connector.connect("192.168.8.11:4646", stream).await?;
+        let stream = connector
+            .connect(self.websocket_url.as_str(), stream)
+            .await?;
         Ok(stream)
     }
 
     /// Opens a query stream
-    pub async fn volga_open_log_query(&self, query: &volga::Query) -> Result<volga::QueryStream> {
-        volga::QueryStream::new(self, query).await
+    pub async fn volga_open_log_query(
+        &self,
+        query: &volga::log_query::Query,
+    ) -> Result<volga::log_query::QueryStream> {
+        volga::log_query::QueryStream::new(self, query).await
     }
 
     /// Try to open a Strongbox Vault
