@@ -5,13 +5,9 @@ use crate::{Error, Result};
 use futures_util::StreamExt;
 use serde::{Deserialize, Serialize};
 
-mod consumer;
-mod log_query;
-mod producer;
-
-pub use consumer::*;
-pub use log_query::*;
-pub use producer::*;
+pub mod consumer;
+pub mod log_query;
+pub mod producer;
 
 type WebSocketStream =
     tokio_tungstenite::WebSocketStream<tokio_native_tls::TlsStream<tokio::net::TcpStream>>;
@@ -34,7 +30,7 @@ pub enum Format {
     #[serde(rename = "json")]
     JSON,
     /// String encoded
-    #[serde(rename = "bytes")]
+    #[serde(rename = "string")]
     String,
 }
 
@@ -44,28 +40,74 @@ impl Default for Format {
     }
 }
 
+/// Behavior when topic doesn't exist
+#[derive(Clone, Copy, Debug)]
+// #[serde(rename_all = "kebab-case")]
+pub enum OnNoExists {
+    Create(CreateOptions),
+    Wait,
+    Fail,
+}
+
+impl Serialize for OnNoExists {
+    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        use serde::ser::SerializeMap;
+        let mut map = serializer.serialize_map(None)?;
+        map.serialize_entry(
+            "on-no-exists",
+            match self {
+                Self::Wait => "wait",
+                Self::Fail => "fail",
+                Self::Create(_) => "create",
+            },
+        )?;
+        if let Self::Create(create_opts) = self {
+            map.serialize_entry("create-options", create_opts)?;
+        }
+
+        map.end()
+    }
+}
+
+/// Local or NAT connection
+#[derive(Clone, Copy, Debug, Serialize, PartialEq)]
+#[serde(rename_all = "kebab-case")]
+pub enum Location {
+    Local,
+    ChildSite,
+    Parent,
+}
+
 /// Volga options for consumers and producers
 #[derive(Clone, Copy, Debug, Serialize)]
-pub struct Options {
-    /// Fail if the topic does not exist
-    pub create: bool,
+#[serde(rename_all = "kebab-case")]
+pub struct CreateOptions {
     /// Number of replicas in the cluster
-    #[serde(rename = "replication-factor")]
     pub replication_factor: u32,
     /// Volga stream persistence
     pub persistence: Persistence,
 
+    /// number of 1Mb chunks
+    pub num_chunks: usize,
+
     /// Volga format
     pub format: Format,
+
+    /// Delete topic after call producers and consumers disconnect
+    pub ephemeral: bool,
 }
 
-impl Default for Options {
+impl Default for CreateOptions {
     fn default() -> Self {
         Self {
-            create: true,
             replication_factor: 1,
             persistence: Persistence::Disk,
             format: Format::default(),
+            num_chunks: 100,
+            ephemeral: false,
         }
     }
 }
@@ -80,9 +122,12 @@ async fn get_binary_response(ws: &mut WebSocketStream) -> Result<Vec<u8>> {
         match resp {
             tokio_tungstenite::tungstenite::Message::Pong(_) => continue,
             tokio_tungstenite::tungstenite::Message::Binary(m) => return Ok(m),
+            tokio_tungstenite::tungstenite::Message::Close(_) => {
+                return Err(Error::Volga(Some("closed".to_string())));
+            }
             msg => {
                 return Err(Error::Volga(Some(format!(
-                    "Unexpected message type: {}",
+                    "Unexpected message type: '{}'",
                     msg
                 ))))
             }
@@ -113,3 +158,21 @@ struct VolgaResponse {
     result: VolgaResult,
     info: Option<String>,
 }
+
+#[cfg(test)]
+mod test {
+    #[test]
+    fn on_no_exists() {
+        let wait = serde_json::to_string(&super::OnNoExists::Wait).unwrap();
+        assert_eq!(&wait, r#"{"on-no-exists":"wait"}"#);
+
+        let fail = serde_json::to_string(&super::OnNoExists::Fail).unwrap();
+        assert_eq!(&fail, r#"{"on-no-exists":"fail"}"#);
+
+        let create = serde_json::to_string(&super::OnNoExists::Create(Default::default())).unwrap();
+        assert_eq!(
+            &create,
+            r#"{"on-no-exists":"create","create-options":{"replication-factor":1,"persistence":"disk","num-chunks":100,"format":"json","ephemeral":false}}"#
+        );
+    }
+}
